#include <bluefruit.h>
#include <Adafruit_NeoPixel.h>

#define NUM_PIXELS 28

#define BUTTON_A_PIN 4
#define BUTTON_B_PIN 5
#define SLIDER_PIN 7
#define PIXEL_PIN 8
#define LED_POWER_PIN 35

Adafruit_NeoPixel strip(NUM_PIXELS, PIXEL_PIN, NEO_GRB + NEO_KHZ800);

int currentPaletteIndex = 0;
int currentSpeed = 5;
int targetSpeed = 5;
unsigned long lastSpeedChangeMillis = 0;
unsigned long lastLightningMillis = 0;
unsigned long lastMinuteMillis = 0;

// Palettes
uint32_t weather_palettes[][10] = {
  {strip.Color(0, 255, 0), strip.Color(0, 128, 0), strip.Color(0, 192, 0), strip.Color(0, 100, 0), strip.Color(0, 139, 139), strip.Color(0, 0, 255), strip.Color(0, 0, 128), strip.Color(0, 0, 192), strip.Color(0, 0, 205), strip.Color(65, 105, 225)},
  {strip.Color(128, 0, 128), strip.Color(0, 0, 255), strip.Color(0, 255, 0), strip.Color(255, 255, 0), strip.Color(255, 165, 0), strip.Color(255, 0, 0)},
  {strip.Color(255, 255, 0), strip.Color(255, 165, 0), strip.Color(255, 223, 0), strip.Color(255, 236, 139)},
  {strip.Color(0, 0, 255), strip.Color(0, 0, 128), strip.Color(0, 0, 192), strip.Color(65, 105, 225)},
  {strip.Color(0, 0, 0), strip.Color(128, 128, 128), strip.Color(47, 79, 79), strip.Color(112, 128, 144)},
  {strip.Color(0, 128, 0), strip.Color(255, 255, 255), strip.Color(255, 136, 0)},
  {strip.Color(255, 0, 0), strip.Color(0, 0, 255), strip.Color(255, 255, 255)},
  {strip.Color(255, 165, 0), strip.Color(0, 0, 0), strip.Color(139, 0, 0)},
  {strip.Color(0, 128, 128), strip.Color(0, 139, 139), strip.Color(0, 205, 205), strip.Color(72, 209, 204)},
  {strip.Color(25, 25, 112), strip.Color(148, 0, 211), strip.Color(72, 61, 139), strip.Color(106, 90, 205)},
  {strip.Color(255, 0, 0), strip.Color(255, 69, 0), strip.Color(255, 99, 71), strip.Color(220, 20, 60)},
  {strip.Color(255, 69, 0), strip.Color(220, 20, 60), strip.Color(255, 160, 122), strip.Color(184, 134, 11)}
};
uint32_t palette_size = sizeof(weather_palettes);

void setup() {
  strip.begin();
  strip.show();
  pinMode(BUTTON_A_PIN, INPUT);
  pinMode(BUTTON_B_PIN, INPUT);
  pinMode(LED_POWER_PIN, OUTPUT);  
  digitalWrite(LED_POWER_PIN, HIGH); 
  pinMode(SLIDER_PIN, INPUT); 
}

void loop() {
  updateSpeed();
  
  if (digitalRead(BUTTON_A_PIN) == LOW) {
    currentPaletteIndex = (currentPaletteIndex + 1) % palette_size; 
    delay(200);
  }

  runChaseEffect();

  checkForLightning();
}

void updateSpeed() {
  unsigned long currentMillis = millis();
  if (currentMillis - lastSpeedChangeMillis >= 60000) {
    if (currentSpeed != targetSpeed) {
      if (currentSpeed < targetSpeed) {
        currentSpeed++;
      } else if (currentSpeed > targetSpeed) {
        currentSpeed--;
      }
    } else {
      targetSpeed = random(1, 11);
    }
    lastSpeedChangeMillis = currentMillis;
  }
}

void runChaseEffect() {
  for (int i = 0; i < strip.numPixels(); i++) {
    uint32_t color = weather_palettes[currentPaletteIndex][i % 10];
    strip.setPixelColor(i, color);
  }
  strip.show();
  delay(1000 / currentSpeed);
}

void checkForLightning() {
  unsigned long currentMillis = millis();
  if (currentMillis - lastLightningMillis >= 10000) {
    int chance = 5 + (2 * currentSpeed);
    if (random(1, 101) <= chance) {
      lastLightningMillis = currentMillis;
    }
  }
}